# -*- coding: utf-8 -*-
# Settings.py
# Copyright © Masaaki Kawata All rights reserved.

import clr
clr.AddReferenceByPartialName("mscorlib")
clr.AddReferenceByPartialName("System")
clr.AddReferenceByPartialName("System.Configuration")
clr.AddReferenceByPartialName("System.Drawing")
clr.AddReferenceByPartialName("System.Windows.Forms")
clr.AddReferenceByPartialName("System.Xml")
clr.AddReferenceByPartialName("Apricot")

from System import String, Boolean, Int32, Int64, Type, Environment, GC
from System.IO import File, Directory, FileInfo, DirectoryInfo, Path
from System.Collections.Generic import List, Dictionary
from System.Configuration import ConfigurationManager, ConfigurationUserLevel, ExeConfigurationFileMap, ConfigurationSaveMode
from System.Diagnostics import Process
from System.Drawing import FontFamily, FontStyle
from System.Globalization import CultureInfo
from System.Reflection import Assembly
from System.Text.RegularExpressions import Regex, RegexOptions
from System.Windows.Forms import Application, Form, MenuItem
from System.Xml import XmlDocument, XmlNode, XmlAttribute, XmlNodeType
from Apricot import Script

program = "notepad.exe"

def onStart(s, e):
	global menuItem
	for form in Application.OpenForms:
		if form.ContextMenu != None:
			if False == form.ContextMenu.MenuItems.Contains(menuItem):
				form.ContextMenu.MenuItems.Add(menuItem)
				form.ContextMenu.Popup += onPopup

def onStop(s, e):
	global menuItem
	for form in Application.OpenForms:
		if form.ContextMenu != None:
			if True == form.ContextMenu.MenuItems.Contains(menuItem):
				form.ContextMenu.MenuItems.Remove(menuItem)
				form.ContextMenu.Popup -= onPopup

def onPopup(s, e):
	global menuItem

	menuItem.MenuItems.Clear()

	config = None
	directoryInfo = DirectoryInfo(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), Assembly.GetEntryAssembly().GetName().Name))
	
	if directoryInfo.Exists:
		fileName = Path.GetFileName(Assembly.GetEntryAssembly().Location)
		regex = Regex(String.Concat(Regex.Escape(".config"), "$"), RegexOptions.IgnoreCase)
		
		for fileInfo in directoryInfo.GetFiles():
			if regex.IsMatch(fileInfo.Extension) and fileName.Equals(Path.GetFileNameWithoutExtension(fileInfo.Name)):
				exeConfigurationFileMap = ExeConfigurationFileMap()
				
				exeConfigurationFileMap.ExeConfigFilename = fileInfo.FullName
				config = ConfigurationManager.OpenMappedExeConfiguration(exeConfigurationFileMap, ConfigurationUserLevel.None)
				
				break
	
	if config == None:
		config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None)
		directoryInfo = None

	if config.HasFile:
		if config.AppSettings.Settings["ActivateThreshold"] != None:
			threshold = Int64.Parse(config.AppSettings.Settings["ActivateThreshold"].Value)
			
			childMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "トーク間隔"
			else:
				childMenuItem.Text = "Talking Interval"

			shortMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				shortMenuItem.Text = "短い"
			else:
				shortMenuItem.Text = "Short"
			
			if threshold == 150000000:
				shortMenuItem.Checked = True
				shortMenuItem.RadioCheck = True
				
			def onShortClick(sender, args):
				config.AppSettings.Settings["ActivateThreshold"].Value = "150000000"
				config.Save(ConfigurationSaveMode.Modified)
			
			shortMenuItem.Click += onShortClick
					
			childMenuItem.MenuItems.Add(shortMenuItem)
			
			middleMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				middleMenuItem.Text = "普通"
			else:
				middleMenuItem.Text = "Middle"
			
			if threshold == 300000000:
				middleMenuItem.Checked = True
				middleMenuItem.RadioCheck = True
				
			def onMiddleClick(sender, args):
				config.AppSettings.Settings["ActivateThreshold"].Value = "300000000"
				config.Save(ConfigurationSaveMode.Modified)
			
			middleMenuItem.Click += onMiddleClick
					
			childMenuItem.MenuItems.Add(middleMenuItem)
			
			longMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				longMenuItem.Text = "長い"
			else:
				longMenuItem.Text = "Long"
			
			if threshold == 600000000:
				longMenuItem.Checked = True
				longMenuItem.RadioCheck = True
				
			def onLongClick(sender, args):
				config.AppSettings.Settings["ActivateThreshold"].Value = "600000000"
				config.Save(ConfigurationSaveMode.Modified)
			
			longMenuItem.Click += onLongClick
					
			childMenuItem.MenuItems.Add(longMenuItem)
			
			menuItem.MenuItems.Add(childMenuItem)
			
		menuItem.MenuItems.Add(MenuItem("-"))
		
		childMenuItem = MenuItem()
		
		if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
			childMenuItem.Text = "テーマ"
		else:
			childMenuItem.Text = "Theme"

		menuItem1 = MenuItem()
		
		if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
			menuItem1.Text = "ドット"
		else:
			menuItem1.Text = "Dots"
			
		menuItem2 = MenuItem()
			
		if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
			menuItem2.Text = "メタルホール"
		else:
			menuItem2.Text = "Metal Hole"
			
		menuItem3 = MenuItem()
		
		if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
			menuItem3.Text = "ノイズ"
		else:
			menuItem3.Text = "Noize"
				
		if config.AppSettings.Settings["BackgroundColor"] != None and config.AppSettings.Settings["BackgroundImage"] != None and config.AppSettings.Settings["TextColor"] != None and config.AppSettings.Settings["LinkColor"]:
			backColor = config.AppSettings.Settings["BackgroundColor"].Value
			backImage = config.AppSettings.Settings["BackgroundImage"].Value
			textColor = config.AppSettings.Settings["TextColor"].Value
			linkColor = config.AppSettings.Settings["LinkColor"].Value
			
			if backColor.Equals("#FFD1D1D1") and backImage.Equals("Assets\\Background-Dots.png") and textColor.Equals("#FFFFFFFF") and linkColor.Equals("#FF33FFFF"):
				menuItem1.Checked = True
				menuItem1.RadioCheck = True
				
			def onClick1(sender, args):
				config.AppSettings.Settings["BackgroundColor"].Value = "#FFD1D1D1"
				config.AppSettings.Settings["BackgroundImage"].Value = "Assets\\Background-Dots.png"
				config.AppSettings.Settings["TextColor"].Value = "#FFFFFFFF"
				config.AppSettings.Settings["LinkColor"].Value = "#FF33FFFF"
				config.Save(ConfigurationSaveMode.Modified)
			
			menuItem1.Click += onClick1
			
			if backColor.Equals("#FF969696") and backImage.Equals("Assets\\Background-MetalHole.png") and textColor.Equals("#FFFFFFFF") and linkColor.Equals("#FF33FFFF"):
				menuItem2.Checked = True
				menuItem2.RadioCheck = True
				
			def onClick2(sender, args):
				config.AppSettings.Settings["BackgroundColor"].Value = "#FF969696"
				config.AppSettings.Settings["BackgroundImage"].Value = "Assets\\Background-MetalHole.png"
				config.AppSettings.Settings["TextColor"].Value = "#FFFFFFFF"
				config.AppSettings.Settings["LinkColor"].Value = "#FF33FFFF"
				config.Save(ConfigurationSaveMode.Modified)
			
			menuItem2.Click += onClick2
			
			if backColor.Equals("#FF1A1A1A") and backImage.Equals("Assets\\Background-Noize.png") and textColor.Equals("#FFFFFFFF") and linkColor.Equals("#FF00C0FF"):
				menuItem3.Checked = True
				menuItem3.RadioCheck = True
				
			def onClick3(sender, args):
				config.AppSettings.Settings["BackgroundColor"].Value = "#FF1A1A1A"
				config.AppSettings.Settings["BackgroundImage"].Value = "Assets\\Background-Noize.png"
				config.AppSettings.Settings["TextColor"].Value = "#FFFFFFFF"
				config.AppSettings.Settings["LinkColor"].Value = "#FF00C0FF"
				config.Save(ConfigurationSaveMode.Modified)
			
			menuItem3.Click += onClick3
		
		childMenuItem.MenuItems.Add(menuItem1)
		childMenuItem.MenuItems.Add(menuItem2)
		childMenuItem.MenuItems.Add(menuItem3)
		menuItem.MenuItems.Add(childMenuItem)
		
		menuItem.MenuItems.Add(MenuItem("-"))
		
		if config.AppSettings.Settings["DropShadow"] != None:
			dropShadow = Boolean.Parse(config.AppSettings.Settings["DropShadow"].Value)
			
			childMenuItem = MenuItem()
			childMenuItem.Checked = dropShadow
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "ドロップシャドウを有効にする"
			else:
				childMenuItem.Text = "Enable Drop Shadow"
			
			def onClick(sender, args):
				sender.Checked = not sender.Checked
				config.AppSettings.Settings["DropShadow"].Value = sender.Checked.ToString()
				config.Save(ConfigurationSaveMode.Modified)
					
			childMenuItem.Click += onClick
			
			menuItem.MenuItems.Add(childMenuItem)
				
		if config.AppSettings.Settings["Reflection"] != None:
			reflection = Boolean.Parse(config.AppSettings.Settings["Reflection"].Value)
			
			childMenuItem = MenuItem()
			childMenuItem.Checked = reflection
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "リフレクションを有効にする"
			else:
				childMenuItem.Text = "Enable Reflection"
			
			def onClick(sender, args):
				sender.Checked = not sender.Checked
				config.AppSettings.Settings["Reflection"].Value = sender.Checked.ToString()
				config.Save(ConfigurationSaveMode.Modified)
					
			childMenuItem.Click += onClick
			
			menuItem.MenuItems.Add(childMenuItem)
			
		menuItem.MenuItems.Add(MenuItem("-"))
				
		if config.AppSettings.Settings["ShowInTaskbar"] != None:
			showInTaskbar = Boolean.Parse(config.AppSettings.Settings["ShowInTaskbar"].Value)
			
			childMenuItem = MenuItem()
			childMenuItem.Checked = showInTaskbar
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "タスクバーに表示する"
			else:
				childMenuItem.Text = "Show in Taskbar"
			
			def onClick(sender, args):
				sender.Checked = not sender.Checked
				config.AppSettings.Settings["ShowInTaskbar"].Value = sender.Checked.ToString()
				config.Save(ConfigurationSaveMode.Modified)
					
			childMenuItem.Click += onClick
			
			menuItem.MenuItems.Add(childMenuItem)
			
		menuItem.MenuItems.Add(MenuItem("-"))
		
		if config.AppSettings.Settings["FontFamily"] != None:
			fontFamilyName = config.AppSettings.Settings["FontFamily"].Value
			
			childMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "フォント"
			else:
				childMenuItem.Text = "Font"
				
			for fontFamily in ["Arial", "Calibri", "Cambria", "Candara", "Constantia", "Corbel", "Courier New", "Geogia", "MS UI Gothic", "Segoe UI", "Tahoma", "Times New Roman", "Verdana", "メイリオ", "ＭＳ ゴシック"]:
				fontMenuItem = MenuItem(fontFamily)
			
				if fontFamily.Equals(fontFamilyName):
					fontMenuItem.Checked = True
					fontMenuItem.RadioCheck = True
					
				def onClick(sender, args):
					try:
						fm = FontFamily(sender.Text)
						
						if fm.IsStyleAvailable(FontStyle.Regular):
							config.AppSettings.Settings["FontFamily"].Value = sender.Text
							config.Save(ConfigurationSaveMode.Modified)
					except:
						return				
				fontMenuItem.Click += onClick
			
				childMenuItem.MenuItems.Add(fontMenuItem)
			
			menuItem.MenuItems.Add(childMenuItem)
			
		if config.AppSettings.Settings["FontSize"] != None:
			fontSize = Int32.Parse(config.AppSettings.Settings["FontSize"].Value)
			
			childMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "フォントサイズ"
			else:
				childMenuItem.Text = "Font Size"
			
			for i in [8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24]:
				fontMenuItem = MenuItem(i.ToString())
				
				if fontSize == i:
					fontMenuItem.Checked = True
					fontMenuItem.RadioCheck = True
					
				def onClick(sender, args):
					config.AppSettings.Settings["FontSize"].Value = sender.Text
					config.Save(ConfigurationSaveMode.Modified)
				
				fontMenuItem.Click += onClick
				
				childMenuItem.MenuItems.Add(fontMenuItem)
				
			menuItem.MenuItems.Add(childMenuItem)
				
			if config.AppSettings.Settings["LineHeight"] != None:
				lineHeight = Int32.Parse(config.AppSettings.Settings["LineHeight"].Value)
				maxLineHeight = fontSize * 2
				
				if maxLineHeight < lineHeight:
					maxLineHeight = lineHeight
				
				childMenuItem2 = MenuItem()
				
				if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
					childMenuItem2.Text = "行間"
				else:
					childMenuItem2.Text = "Line Height"
				
				for i in range(fontSize, maxLineHeight + 1):
					lineHeightMenuItem = MenuItem(i.ToString())
					
					if lineHeight == i:
						lineHeightMenuItem.Checked = True
						lineHeightMenuItem.RadioCheck = True
						
					def onClick(sender, args):
						config.AppSettings.Settings["LineHeight"].Value = sender.Text
						config.Save(ConfigurationSaveMode.Modified)
					
					lineHeightMenuItem.Click += onClick
					
					childMenuItem2.MenuItems.Add(lineHeightMenuItem)
			
				menuItem.MenuItems.Add(childMenuItem2)
				
		menuItem.MenuItems.Add(MenuItem("-"))
		
		if config.AppSettings.Settings["FrameRate"] != None:
			frameRate = Int32.Parse(config.AppSettings.Settings["FrameRate"].Value)
			
			childMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "フレームレート"
			else:
				childMenuItem.Text = "Frame Rate"
			
			for i in [24, 30, 60]:
				frameRateMenuItem = MenuItem(i.ToString())
				
				if frameRate == i:
					frameRateMenuItem.Checked = True
					frameRateMenuItem.RadioCheck = True
					
				def onClick(sender, args):
					config.AppSettings.Settings["FrameRate"].Value = sender.Text
					config.Save(ConfigurationSaveMode.Modified)
				
				frameRateMenuItem.Click += onClick
				
				childMenuItem.MenuItems.Add(frameRateMenuItem)
				
			menuItem.MenuItems.Add(childMenuItem)
			
		menuItem.MenuItems.Add(MenuItem("-"))
		
		if config.AppSettings.Settings["Subscriptions"] != None:
			path = config.AppSettings.Settings["Subscriptions"].Value
			regex = Regex(String.Concat(Regex.Escape(".opml"), "$"), RegexOptions.IgnoreCase)
			fileList = List[String]()
			
			childMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "フィード"
			else:
				childMenuItem.Text = "Subscriptions"
			
			def onEdit(sender, args):
				global program
				if Directory.Exists(path):
					for fileInfo in DirectoryInfo(path).GetFiles():
						if regex.IsMatch(fileInfo.Extension):
							Process.Start(fileInfo.FullName)
				elif File.Exists(path):
					if String.IsNullOrEmpty(program):
						Process.Start(path)
					else:
						Process.Start(program, path)
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.MenuItems.Add(MenuItem("フィードの編集...", onEdit))
			else:
				childMenuItem.MenuItems.Add(MenuItem("Edit...", onEdit))
			childMenuItem.MenuItems.Add(MenuItem("-"))
			
			if directoryInfo != None:
				fileInfo = FileInfo(Path.Combine(directoryInfo.FullName, path))
			
				if fileInfo.Exists:
					path = fileInfo.FullName
					
			if Directory.Exists(path):
				for fileInfo in DirectoryInfo(path).GetFiles():
					if regex.IsMatch(fileInfo.Extension):
						fileList.Add(fileInfo.FullName)

			elif File.Exists(path):
				fileList.Add(path)
			
			dictionary = Dictionary[String, String]()
			
			def onClick(sender, args):
				if dictionary.ContainsKey(sender.Text):
					Process.Start(dictionary[sender.Text])
			
			def parseOutline(m, n):
				if not n.HasChildNodes:
					return None

				for xmlNode in n.ChildNodes:
					if xmlNode.Name.Equals("outline"):
						text = None
						xmlUrl = None
						htmlUrl = None

						for xmlAttribute in xmlNode.Attributes:
							if xmlAttribute.Name.Equals("title") or xmlAttribute.Name.Equals("text"):
								text = xmlAttribute.Value
							elif xmlAttribute.Name.Equals("xmlUrl"):
								xmlUrl = xmlAttribute.Value
							elif xmlAttribute.Name.Equals("htmlUrl"):
								htmlUrl = xmlAttribute.Value
						
						if not String.IsNullOrEmpty(text):
							if String.IsNullOrEmpty(xmlUrl):
								childMenuItem = MenuItem(text)
								m.MenuItems.Add(childMenuItem)
								parsedMenuItem = parseOutline(childMenuItem, xmlNode)
								if not parsedMenuItem == None:
									m.MenuItems.Add(parsedMenuItem)
							elif not String.IsNullOrEmpty(xmlUrl):
								m.MenuItems.Add(MenuItem(text, onClick))
								
								if not String.IsNullOrEmpty(htmlUrl):
									if dictionary.ContainsKey(text):
										dictionary[text] = htmlUrl
									else:
										dictionary.Add(text, htmlUrl)

				return m
				
			for xmlPath in fileList:
				doc = XmlDocument()
				doc.Load(xmlPath)

				for xmlNode in doc.SelectNodes("/opml/body"):
					parseOutline(childMenuItem, xmlNode)
				
			menuItem.MenuItems.Add(childMenuItem)
		
		if config.AppSettings.Settings["Timeout"] != None:
			timeout = Int32.Parse(config.AppSettings.Settings["Timeout"].Value)
			
			childMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "タイムアウト"
			else:
				childMenuItem.Text = "Timeout"
			
			for i in [15000, 30000, 60000, 120000, 180000]:
				timeMenuItem = MenuItem(i.ToString())
				
				if timeout == i:
					timeMenuItem.Checked = True
					timeMenuItem.RadioCheck = True
					
				def onClick(sender, args):
					config.AppSettings.Settings["Timeout"].Value = sender.Text
					config.Save(ConfigurationSaveMode.Modified)
				
				timeMenuItem.Click += onClick
				
				childMenuItem.MenuItems.Add(timeMenuItem)
					
			menuItem.MenuItems.Add(childMenuItem)
		
		if config.AppSettings.Settings["Cache"] != None:
			path = config.AppSettings.Settings["Cache"].Value
			
			if directoryInfo != None:
				path = Path.Combine(directoryInfo.FullName, path);
				
			childMenuItem = MenuItem()
			
			if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
				childMenuItem.Text = "キャッシュをクリア"
			else:
				childMenuItem.Text = "Clear Cache"
				
			def onClick(sender, args):
				if Directory.Exists(path):
					for fileName in Directory.GetFiles(path):
						File.Delete(fileName)
			
			childMenuItem.Click += onClick
			
			menuItem.MenuItems.Add(MenuItem("-"))
			menuItem.MenuItems.Add(childMenuItem)
		
		childMenuItem = MenuItem()
		
		if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
			childMenuItem.Text = "GCを強制的に実行"
		else:
			childMenuItem.Text = "Force Garbage Collection"
			
		def onClick(sender, args):
			GC.Collect()
		
		childMenuItem.Click += onClick
		
		menuItem.MenuItems.Add(childMenuItem)

menuItem = MenuItem()
if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
	menuItem.Text = "オプション"
else:
	menuItem.Text = "Settings"
Script.Instance.Start += onStart
Script.Instance.Stop += onStop