# -*- coding: utf-8 -*-
# Social.py
# Copyright © Masaaki Kawata All rights reserved.

import clr
clr.AddReferenceByPartialName("mscorlib")
clr.AddReferenceByPartialName("System")
clr.AddReferenceByPartialName("System.Windows.Forms")
clr.AddReferenceByPartialName("System.Web")
clr.AddReferenceByPartialName("System.Xml")
clr.AddReferenceByPartialName("Apricot")

from System import Object, Boolean, Byte, Int32, String, Uri, DateTime, TimeSpan, Type, Environment
from System.IO import File, Directory, FileInfo, DirectoryInfo, Path, StreamReader
from System.Collections.Generic import List, Dictionary, KeyValuePair
from System.Globalization import CultureInfo
from System.Diagnostics import Process, Trace
from System.Reflection import Assembly
from System.Security.Cryptography import SHA1CryptoServiceProvider
from System.Text import StringBuilder, Encoding
from System.Text.RegularExpressions import Regex, Match
from System.Threading import Monitor
from System.Timers import Timer
from System.Net import NetworkCredential, WebRequest, WebResponse, WebClient, HttpRequestHeader
from System.Net.NetworkInformation import NetworkInterface, PhysicalAddress
from System.Windows.Forms import Application, Form, MenuItem
from System.Web import HttpUtility
from System.Xml import XmlDocument, XmlNode, XmlAttribute, XmlNodeType
from Apricot import Agent, Balloon, Script, Entry, Message, Word

def update():
	global syncRoot, dateTime, recentEntryList

	try:
		if not NetworkInterface.GetIsNetworkAvailable():
			return
		
		entryList = List[Entry]()
		request = WebRequest.Create("http://social.apricotan.net/entries?limit=25")
		response = request.GetResponse()
		s = response.GetResponseStream()
		doc = XmlDocument()
		doc.Load(s)
		
		for entryXmlNode in doc.SelectNodes("/ArrayOfEntry/Entry"):
			newEntry = Entry()
			
			for xmlNode in entryXmlNode.ChildNodes:
				if xmlNode.Name.Equals("Resource"):
					newEntry.Resource = Uri(xmlNode.InnerText)
				elif xmlNode.Name.Equals("Title"):
					newEntry.Title = xmlNode.InnerText
				elif xmlNode.Name.Equals("Created"):
					newEntry.Created = DateTime.Parse(xmlNode.InnerText)
				elif xmlNode.Name.Equals("Modified"):
					newEntry.Modified = DateTime.Parse(xmlNode.InnerText)
				elif xmlNode.Name.Equals("Image"):
					newEntry.Image = Uri(xmlNode.InnerText)
				elif xmlNode.Name.Equals("Tags"):
					for childXmlNode in xmlNode.ChildNodes:
						if childXmlNode.Name.Equals("string"):
							newEntry.Tags.Add(childXmlNode.InnerText)
			
			try:
				Monitor.Enter(syncRoot)
				
				if newEntry.Resource != None and String.IsNullOrEmpty(newEntry.Title) == False and entryList.Exists(lambda x: x.Resource.Equals(newEntry.Resource)) == False:
					entryList.Add(newEntry)
			finally:
				Monitor.Exit(syncRoot)
		
		if entryList.Count > 0:
			try:
				Monitor.Enter(syncRoot)
				
				newEntryList = List[Entry]()
				nowDateTime = DateTime.Now
				
				for entry in entryList:
					if entry.Modified > dateTime and entry.Modified <= nowDateTime:
						newEntryList.Add(entry)
				
				dateTime = nowDateTime
				
				if entryList.Count > 10:
					entryList.RemoveRange(10, entryList.Count - 10)
					entryList.TrimExcess()
					
				if newEntryList.Count > 0:
					if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
						Script.Instance.Suggest("ソーシャル", newEntryList)
					else:
						Script.Instance.Suggest("Social", newEntryList)
		
					for entry in newEntryList:
						if entry.HasTags:
							if Script.Instance.TryEnqueue(Script.Instance.Prepare(Script.Instance.GetSequences("Activate"), None, entry.Tags)):
								break
					
				recentEntryList.Clear()
				recentEntryList.AddRange(entryList)
			finally:
				Monitor.Exit(syncRoot)
			
		if s != None:
			s.Close()
			
		if response != None:
			response.Close()

	except Exception, e:
		Trace.WriteLine(e.clsException.Message)
		Trace.WriteLine(e.clsException.StackTrace)

def onTimedEvent(timer, e):
	update()
			
	timer.Interval = 1000 * 60 * 5
	timer.AutoReset = True
	timer.Start()

def onStart(s, e):
	global agentList, menuItem, timer

	timer.Interval = 1000 * 60 * 1
	timer.AutoReset = False
	timer.Start()
	for form in Application.OpenForms:
		if form.GetType().Name.Equals("Agent"):
			agentList.Add(form)
		if form.ContextMenu != None:
			if False == form.ContextMenu.MenuItems.Contains(menuItem):
				form.ContextMenu.MenuItems.Add(menuItem)
				form.ContextMenu.Popup += onPopup

def onStop(s, e):
	global agentList, menuItem, timer

	timer.Stop()
	agentList.Clear()
	menuItem.MenuItems.Clear()
	for form in Application.OpenForms:
		if form.ContextMenu != None:
			if True == form.ContextMenu.MenuItems.Contains(menuItem):
				form.ContextMenu.MenuItems.Remove(menuItem)
				form.ContextMenu.Popup -= onPopup

def onClick(s, e):
	global dictionary

	if dictionary.ContainsKey(s.Text):
		Process.Start(dictionary[s.Text].Resource.ToString())

def getUserID():
	stringBuilder = StringBuilder()
	nic = NetworkInterface.GetAllNetworkInterfaces()
	
	for i in range(0, nic.Length):
		if i == NetworkInterface.LoopbackInterfaceIndex:
			continue

		addr = nic[i].GetPhysicalAddress()

		if addr.GetAddressBytes().Length == 6:
			sha1 = SHA1CryptoServiceProvider()

			for b in sha1.ComputeHash(Encoding.UTF8.GetBytes(String.Concat(addr.ToString(), Environment.MachineName, Environment.UserName))):
				stringBuilder.Append(b.ToString("x2"))

			break

	return stringBuilder.ToString()

def onPing(s, e):
	global dictionary, timer
	
	try:
		if not NetworkInterface.GetIsNetworkAvailable():
			return
		
		if dictionary.ContainsKey(s.Text):
			entry = dictionary[s.Text]
			
			if entry.Resource != None and String.IsNullOrEmpty(entry.Title) == False:
				stringBuilder = StringBuilder()
				
				stringBuilder.AppendFormat("http://social.apricotan.net/ping?resource={0}&title={1}&author={2}&created={3}&modified={4}", HttpUtility.UrlEncode(entry.Resource.ToString()), HttpUtility.UrlEncode(entry.Title), HttpUtility.UrlEncode(getUserID()), HttpUtility.UrlEncode(entry.Created.ToString("s")), HttpUtility.UrlEncode(DateTime.Now.ToString("s")))
				
				if entry.Image != None:
					stringBuilder.AppendFormat("&image={0}", HttpUtility.UrlEncode(entry.Image.ToString()))

				if entry.HasTags:
					sb = StringBuilder()
					for tag in entry.Tags:
						if sb.Length > 0:
							sb.AppendFormat(" {0}", tag)
						else:
							sb.Append(tag)
					stringBuilder.AppendFormat("&tags={0}", HttpUtility.UrlEncode(sb.ToString()))

				client = WebClient()
				client.Headers.Add(HttpRequestHeader.ContentType, "text/xml")
				client.UploadStringAsync(Uri(stringBuilder.ToString()), "POST", String.Empty)
				
				timer.Stop()
				timer.Interval = 1000 * 10
				timer.AutoReset = False
				timer.Start()

	except Exception, e:
		Trace.WriteLine(e.clsException.Message)
		Trace.WriteLine(e.clsException.StackTrace)

def onPopup(s, e):
	global syncRoot, agentList, menuItem, dictionary

	maxLength = 20

	dictionary.Clear()
	menuItem.MenuItems.Clear()

	pingMenuItem = MenuItem()
	
	if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
		pingMenuItem.Text = "共有のお気に入りに追加"
	else:
		pingMenuItem.Text = "Ping"
	
	menuItem.MenuItems.Add(pingMenuItem)
	menuItem.MenuItems.Add(MenuItem("-"))
	
	recentMenuItemList = List[MenuItem]()
	
	try:
		Monitor.Enter(syncRoot)
		
		for entry in recentEntryList:
			title = entry.Title
			
			if title.Length > maxLength:
				title = String.Concat(title.Remove(maxLength, title.Length - maxLength), "...")
			
			if dictionary.ContainsKey(title) == False:
				dictionary.Add(title, entry)
				recentMenuItemList.Add(MenuItem(title, onClick))
	finally:
		Monitor.Exit(syncRoot)

	if recentMenuItemList.Count > 0:
		for recentMenuItem in recentMenuItemList:
			menuItem.MenuItems.Add(recentMenuItem)
	else:
		naMenuItem = MenuItem("N/A")
		naMenuItem.Enabled = False
		menuItem.MenuItems.Add(naMenuItem)
		
	pingEntryList = List[Entry]()
	
	for agent in agentList:
		messageList = List[Message](agent.Balloon.Messages)
		messageList.Reverse()
		for message in messageList:
			if message.HasAttachments:
				for entry in message.Attachments:
					if entry.Resource != None and String.IsNullOrEmpty(entry.Title) == False and pingEntryList.Exists(lambda x: x.Resource.Equals(entry.Resource)) == False:
						pingEntryList.Add(entry)
	
	if pingEntryList.Count == 0:
		naMenuItem = MenuItem("N/A")
		naMenuItem.Enabled = False
		pingMenuItem.MenuItems.Add(naMenuItem)
	else:
		for entry in pingEntryList:
			title = entry.Title
						
			if title.Length > maxLength:
				title = String.Concat(title.Remove(maxLength, title.Length - maxLength), "...")

			if dictionary.ContainsKey(title) == False:
				dictionary.Add(title, entry)
			
			pingMenuItem.MenuItems.Add(MenuItem(title, onPing))

syncRoot = Object()
dateTime = DateTime.Now - TimeSpan(12, 0, 0)
dictionary = Dictionary[String, Entry]()
agentList = List[Agent]()
recentEntryList = List[Entry]()
menuItem = MenuItem()
if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
	menuItem.Text = "ソーシャル"
else:
	menuItem.Text = "Social"
timer = Timer()
timer.Elapsed += onTimedEvent
timer.Enabled = False
Script.Instance.Start += onStart
Script.Instance.Stop += onStop