# -*- coding: utf-8 -*-
# Timer.py
# Copyright © Masaaki Kawata All rights reserved.

import clr
clr.AddReferenceByPartialName("mscorlib")
clr.AddReferenceByPartialName("System")
clr.AddReferenceByPartialName("System.Windows.Forms")
clr.AddReferenceByPartialName("Apricot")

from System import String, DateTime, TimeSpan, Math
from System.Collections.Generic import List
from System.Globalization import CultureInfo
from System.Timers import Timer
from System.Windows.Forms import Application, Form, MenuItem
from Apricot import Entry, Script

def onStart(s, e):
	global menuItem
	for form in Application.OpenForms:
		if form.ContextMenu != None:
			if False == form.ContextMenu.MenuItems.Contains(menuItem):
				for i in range(1, 6):
					if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
						menuItem.MenuItems.Add(MenuItem(i.ToString() + "分", onClick))
					else:
						if i == 1:
							menuItem.MenuItems.Add(MenuItem(i.ToString() + " minute", onClick))
						else:
							menuItem.MenuItems.Add(MenuItem(i.ToString() + " minutes", onClick))
				form.ContextMenu.MenuItems.Add(menuItem)

def onStop(s, e):
	global menuItem
	menuItem.MenuItems.Clear()
	for form in Application.OpenForms:
		if form.ContextMenu != None:
			if True == form.ContextMenu.MenuItems.Contains(menuItem):
				form.ContextMenu.MenuItems.Remove(menuItem)

def onTimedEvent(timer, e):
	global ts
	ts = ts.Add(TimeSpan(0, 0, 0, 0, timer.Interval))
	entry = Entry()
	if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
		entry.Title = String.Format("予定の{0}分前になりました", Math.Abs(ts.Minutes).ToString())
	else:
		if Math.Abs(ts.Minutes) == 1:
			entry.Title = String.Format("{0} minute ago.", Math.Abs(ts.Minutes).ToString())
		else:
			entry.Title = String.Format("{0} minutes ago.", Math.Abs(ts.Minutes).ToString())
	if ts.TotalMilliseconds >= 0:
		timer.Stop()
		ts = TimeSpan(0)
		if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
			entry.Title = "予定の時間になりました"
		else:
			entry.Title= "Time Passing."
	entryList = List[Entry]()
	entryList.Add(entry)
	Script.Instance.Alert(entryList)

def onClick(s, e):
	global timer, ts
	i = 1
	for childMenuItem in menuItem.MenuItems:
		if s == childMenuItem:
			ts = TimeSpan(0, -1 * i, 0)
			timer.Interval = 1000 * 60
			timer.AutoReset = True
			timer.Start()
		i = i + 1

ts = TimeSpan(0)
timer = Timer()
timer.Elapsed += onTimedEvent
timer.Enabled = False
menuItem = MenuItem()
if CultureInfo.CurrentCulture.Equals(CultureInfo.GetCultureInfo("ja-JP")):
	menuItem.Text = "タイマー"
else:
	menuItem.Text = "Timer"

Script.Instance.Start += onStart
Script.Instance.Stop += onStop